/*
 *	linux/fs/rbf/utils.c
 *	RBF File System misc utilities
 *
 *	Copyright (C) 2001 Andrew Cannon <ajc@gmx.net>
 * 	
 *  This file is subject to the terms and conditions of the
 *  GNU General Public License. See the file COPYING in the
 *  main directory of this archive for more details.
 */

#include "rbf_defs.h"

#undef DEBUG
#define DEBUG

#ifndef DEBUG
#undef DPRINTK
#define DPRINTK(x...)
#endif

static short months[13] = { 0, -1, 30, 58, 89, 119, 150, 180, 211, 242, 272, 303, 333 };

/*
 *	Read RBF time (3 or 5 byte) and return time_t
 */
time_t rbf_rdtime(const u8 *p, int n)
{
	int yr = p[0] - 70;
	time_t acc = yr * 365;
	
	acc += (yr + 2 - (p[1] < 3)) / 4;		/* add leap days */
	acc += months[p[1]] + p[2];
	if (n == 3)
		return acc * 86400;
	return ((acc * 24 + p[3]) * 60 + p[4]) * 60;
}

/*
 *	Write RBF time (3 or 5 byte)
 */
void rbf_wrtime(u8 *p, int n, time_t t)
{
	int days = t / 86400;
	int leaps = days / (365 * 4 + 1);
	int ldays = days % (365 * 4 + 1);
	int lpost = ldays > (365 + 365 + 31 + 29 - 1);
	int feb29 = ldays == (365 + 365 + 31 + 29 - 1);
	int m;
	
	p[0] = 70 + leaps * 4 + (ldays - lpost) / 365;
	ldays = (ldays - lpost) % 365;
	for (m = 12; m > 0; m--)
		if (ldays > months[m])
			break;
	p[1] = m;
	p[2] = ldays - months[m] + feb29;
	
	if (n == 5) {
		int secs = t % 86400;
		
		p[3] = secs / 3600;
		p[4] = (secs % 3600) / 60;
	}
}


/*
 *	Read directory entry filename into buffer & return length
 */
int rbf_readname(char *buf, const char *s)
{
	int i;
	
	for (i = 0; i < RBF_NAMELEN; i++) {
		if (s[i] == 0)
			break;
		if (s[i] & 0x80) {
			buf[i] = s[i] & 0x7f;
			i += 1;
			break;
		}
		buf[i] = s[i];
	}
#ifndef NODEBUG
	if (i < RBF_NAMELEN)
		buf[i] = 0;	/* for printk */
#endif
	return i;
}


/*
 *	Validate RBF filename
 */
int rbf_validate_name(struct super_block *s, const char *name, int namelen)
{
	int i, c, got_alnum = 0;
	
	if (namelen == 0)
		return -ENOENT;
	if (namelen > RBF_NAMELEN)
		return -ENAMETOOLONG;
	if (s_su_nonamecheck)
		return 0;
	for (i = 0; i < namelen; i++) {
		c = name[i];
		
		if ((c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c >= '0' && c <= '9'))
			got_alnum = 1;
		else if (c != '_' && c != '.' && c != '$')
			return -EINVAL;
	}
	if (!got_alnum)
		return -EINVAL;
	return 0;
}
